@@ -118,15 +118,15 @@
 #'@seealso See \code{\link{wilderSum}}, which is used in calculating a Welles
 #'Wilder type MA.
 #'@references The following site(s) were used to code/document this
-#'indicator:\cr \url{http://www.fmlabs.com/reference/ExpMA.htm}\cr
-#'\url{http://www.fmlabs.com/reference/WeightedMA.htm}\cr
-#'\url{http://www.fmlabs.com/reference/DEMA.htm}\cr
-#'\url{http://www.fmlabs.com/reference/T3.htm}\cr
+#'indicator:\cr \url{https://www.fmlabs.com/reference/ExpMA.htm}\cr
+#'\url{https://www.fmlabs.com/reference/WeightedMA.htm}\cr
+#'\url{https://www.fmlabs.com/reference/DEMA.htm}\cr
+#'\url{https://www.fmlabs.com/reference/T3.htm}\cr
 #'\url{https://www.linnsoft.com/techind/evwma-elastic-volume-weighted-moving-average}\cr
-#'\url{http://www.fmlabs.com/reference/ZeroLagExpMA.htm}\cr
-#'\url{http://www.fmlabs.com/reference/VIDYA.htm}\cr
-#'\url{http://www.traderslog.com/hullmovingaverage}\cr
-#'\url{http://www.arnaudlegoux.com/}\cr
+#'\url{https://www.fmlabs.com/reference/ZeroLagExpMA.htm}\cr
+#'\url{https://www.fmlabs.com/reference/VIDYA.htm}\cr
+#'\url{https://www.traderslog.com/hullmovingaverage}\cr
+#'\url{https://web.archive.org/web/20180222085959/http://arnaudlegoux.com/}\cr
 #'@keywords ts
 #'@examples
 #'
@@ -154,7 +154,6 @@
 #' tail( EMA(x[ 1:100],10), 1 )
 #' 
 #'@rdname MovingAverages
-#'@export
 "SMA" <-
 function(x, n=10, ...) {
 
@@ -172,7 +171,6 @@ function(x, n=10, ...) {
 #-------------------------------------------------------------------------#
 
 #'@rdname MovingAverages
-#'@export
 "EMA" <-
 function (x, n=10, wilder=FALSE, ratio=NULL, ...) {
 
@@ -181,30 +179,14 @@ function (x, n=10, wilder=FALSE, ratio=NULL, ...) {
   x <- try.xts(x, error=as.matrix)
   if( n < 1 || n > NROW(x) )
     stop(sprintf("n = %d is outside valid range: [1, %d]", n, NROW(x)))
-  if(NCOL(x) > 1) {
-    stop("ncol(x) > 1. EMA only supports univariate 'x'")
-  }
-  if( any(nNonNA <- n > colSums(!is.na(x))) )
-    stop("n > number of non-NA values in column(s) ",
-         paste(which(nNonNA), collapse=", "))
 
-  # Check for non-leading NAs
-  # Leading NAs are handled in the C code
-  x.na <- naCheck(x, n)
-  
   # If ratio is specified, and n is not, set n to approx 'correct'
   # value backed out from ratio
   if(missing(n) && !missing(ratio))
-    n <- trunc(2/ratio - 1)
-
-  # Determine decay ratio
-  if(is.null(ratio)) {
-    if(wilder) ratio <- 1/n
-    else       ratio <- 2/(n+1)
-  }
+    n <- NULL
 
   # Call C routine
-  ma <- .Call("ema", x, n, ratio, PACKAGE = "TTR")
+  ma <- .Call("ema", x, n, ratio, isTRUE(wilder), PACKAGE = "TTR")
 
   ma <- reclass(ma,x)
   
@@ -219,19 +201,31 @@ function (x, n=10, wilder=FALSE, ratio=NULL, ...) {
 #-------------------------------------------------------------------------#
 
 #'@rdname MovingAverages
-#'@export
 "DEMA" <-
 function(x, n=10, v=1, wilder=FALSE, ratio=NULL) {
 
   # Double Exponential Moving Average
   # Thanks to John Gavin for the v-factor generalization
 
+  x <- try.xts(x, error=as.matrix)
+  if( n < 1 || n > NROW(x) )
+    stop(sprintf("n = %d is outside valid range: [1, %d]", n, NROW(x)))
+  if(NCOL(x) > 1) {
+    stop("ncol(x) > 1. DEMA only supports univariate 'x'")
+  }
   if(v < 0 || v > 1) {
     stop("Please ensure 0 <= v <= 1")
   }
 
-  dema <- (1 + v) * EMA(x,n,wilder,ratio) -
-    EMA(EMA(x,n,wilder,ratio),n,wilder,ratio) * v
+  if(missing(n) && !missing(ratio))
+    n <- NULL
+
+  # Call C routine
+  ma1 <- .Call("ema", x, n, ratio, isTRUE(wilder), PACKAGE = "TTR")
+  d <- .Call("ema", ma1, n, ratio, isTRUE(wilder), PACKAGE = "TTR")
+
+  dema <- (1 + v) * ma1 - d * v
+  dema <- reclass(dema, x)
 
   if(!is.null(dim(dema))) {
     colnames(dema) <- "DEMA"
@@ -243,7 +237,6 @@ function(x, n=10, v=1, wilder=FALSE, ratio=NULL) {
 #-------------------------------------------------------------------------#
 
 #'@rdname MovingAverages
-#'@export
 "WMA" <-
 function(x, n=10, wts=1:n, ...) {
 
@@ -273,31 +266,22 @@ function(x, n=10, wts=1:n, ...) {
   
   if( NROW(wts) == n ) {
     
-    x <- na.omit(x)
     NAs <- NAx
 
     if( any(is.na(wts)) )
       stop("'wts' vector of length 'n' cannot have NA values")
 
-    # Call Fortran routine
-    ma <- .Fortran( "wma", ia = as.double(x),
-                           lia = as.integer(NROW(x)),
-                           wts = as.double(wts),
-                           n = as.integer(n),
-                           oa = as.double(x),
-                           loa = as.integer(NROW(x)),
-                           PACKAGE = "TTR",
-                           DUP = TRUE )$oa
-   
+    # Call C routine
+    ma <- .Call("wma", x, wts, n, PACKAGE = "TTR")
+
   } else {
     
-    xw <- na.omit( cbind(x, wts) )
+    xw <- cbind(x, wts)
     ma <- runSum( xw[,1]*xw[,2], n) / runSum(xw[,2], n)
   }
 
   # replace 1:(n-1) with NAs and prepend NAs from original data
   ma[1:(n-1)] <- NA
-  ma <- c( rep( NA, NAs ), ma )
 
   if(!is.null(dim(ma))) {
     colnames(ma) <- "WMA"
@@ -309,7 +293,6 @@ function(x, n=10, wts=1:n, ...) {
 #-------------------------------------------------------------------------#
 
 #'@rdname MovingAverages
-#'@export
 "EVWMA" <-
 function(price, volume, n=10, ...) {
 
@@ -351,7 +334,6 @@ function(price, volume, n=10, ...) {
 #-------------------------------------------------------------------------#
 
 #'@rdname MovingAverages
-#'@export
 "ZLEMA" <-
 function (x, n=10, ratio=NULL, ...) {
 
@@ -362,37 +344,14 @@ function (x, n=10, ratio=NULL, ...) {
     stop("ncol(x) > 1. ZLEMA only supports univariate 'x'")
   }
   
-  # Count NAs, ensure they're only at beginning of data, then remove.
-  NAs <- sum( is.na(x) )
-  if( NAs > 0 ) {
-    if( any( is.na(x[-(1:NAs)]) ) )
-      stop("Series contains non-leading NAs")
-  }
-  x   <- na.omit(x)
+  # If ratio is specified, and n is not, set n to approx 'correct'
+  # value backed out from ratio
+  if(missing(n) && !missing(ratio))
+    n <- NULL
 
-  # Initialize ma vector
-  ma <- rep(1, NROW(x))
-  ma[n] <- mean(x[1:n])
+  # Call C routine
+  ma <- .Call("zlema", x, n, ratio, PACKAGE = "TTR")
 
-  # Determine decay ratio
-  if(is.null(ratio)) {
-    ratio <- 2/(n+1)
-  }
-
-  # Call Fortran routine
-  ma <- .Fortran( "zlema", ia = as.double(x),
-                           lia = as.integer(NROW(x)),
-                           n = as.integer(n),
-                           oa = as.double(ma),
-                           loa = as.integer(NROW(ma)),
-                           ratio = as.double(ratio),
-                           PACKAGE = "TTR",
-                           DUP = TRUE )$oa
-
-  # replace 1:(n-1) with NAs and prepend NAs from original data
-  ma[1:(n-1)] <- NA
-  ma <- c( rep( NA, NAs ), ma ) 
-  
   if(!is.null(dim(ma))) {
     colnames(ma) <- "ZLEMA"
   }
@@ -403,7 +362,6 @@ function (x, n=10, ratio=NULL, ...) {
 #-------------------------------------------------------------------------#
 
 #'@rdname MovingAverages
-#'@export VWAP VWMA
 "VWAP" <- "VWMA" <-
 function(price, volume, n=10, ...) {
 
@@ -422,7 +380,6 @@ function(price, volume, n=10, ...) {
 #-------------------------------------------------------------------------#
 
 #'@rdname MovingAverages
-#'@export
 "VMA" <-
 function (x, w, ratio=1, ...) {
 
@@ -456,19 +413,21 @@ function (x, w, ratio=1, ...) {
 #-------------------------------------------------------------------------#
 
 #'@rdname MovingAverages
-#'@export
 "HMA" <-
 function(x, n=20, ...) {
 
   # Hull Moving Average
 
-  reclass(WMA(2*WMA(x, n=n/2, ...) - WMA(x, n=n, ...), n=trunc(sqrt(n)), ...), x)
+  madiff <- 2 * WMA(x, n = trunc(n / 2), ...) - WMA(x, n = n, ...)
+
+  hma <- WMA(madiff, n = trunc(sqrt(n)), ...)
+
+  reclass(hma, x)
 }
 
 #-------------------------------------------------------------------------#
 
 #'@rdname MovingAverages
-#'@export
 "ALMA" <-
 function(x, n=9, offset=0.85, sigma=6, ...) {
 
@@ -486,6 +445,7 @@ function(x, n=9, offset=0.85, sigma=6, ...) {
   sumWeights <- sum(wts)
   if(sumWeights != 0)
     wts <- wts/sumWeights
-  alma <- rollapply(x, width=n, FUN=function(xx) sum(xx*wts), align="right")
+  alma <- rollapply(x, width=n, FUN=function(xx) sum(xx*wts),
+                    fill=NA, align="right")
   reclass(alma, x)
 }
